import 'package:jwt_decoder/jwt_decoder.dart';
import 'package:nui_core/code/nui_code_util.dart';

class NUIAuthAccessToken{
  NUIAuthAccessToken({required this.token, required this.refreshToken}){
    Map<String, dynamic> decodedToken = JwtDecoder.decode(token);
    claims = decodedToken;
    payload = decodedToken["user"];
  }
  final String token;
  final String refreshToken;
  late Map<String, dynamic> claims;
  Map<String, dynamic>? payload;

  String getUserId(){
    return claims["sub"];
  }

  bool isExpired({int leeway = 0}){
    int? exp = claims["exp"];
    int? iat = claims["iat"];
    int todayTime = DateTime.now().millisecondsSinceEpoch;
    logNUI("NUIAuthToken", "exp: $exp, iat: $iat, todayTime: $todayTime");
    int futureToday = todayTime + leeway * 1000;
    int pastToday = todayTime - leeway * 1000;
    bool expValid = exp == null || pastToday <= exp*1000;
    bool iatValid = iat == null || futureToday >= iat*1000;
    return !expValid || !iatValid;
  }

  DateTime tokenExpiryDate(){
    return JwtDecoder.getExpirationDate(token);
  }

  Duration timeRemainingToExpired(){
    return JwtDecoder.getRemainingTime(token);
  }


}