import 'package:flutter/cupertino.dart';
import 'package:nui_auth/domain/nui_user_activity.dart';
import 'package:nui_auth/domain/nui_user_activity.dart';
import 'package:nui_core/data/nui_ent.dart';

enum NUIAuthModule {
  AUTH,
  PUSH,
  FORM,
}

extension NUIAuthModuleExt on NUIAuthModule{
  String? value(){
    switch(this){
      case NUIAuthModule.AUTH: return "Authentication";
      case NUIAuthModule.PUSH: return "Push Notification";
      case NUIAuthModule.FORM: return "Form";
      default: return null;
    }
  }
}

class NUIUserActivity extends NUIEnt<NUIUserActivity>{
  static const int STATUS_ERROR = 0;
  static const int STATUS_SUCCESS = 1;
  NUIUserActivity({required this.module, required this.action, this.extras, this.status = STATUS_SUCCESS, required this.time});
  NUIUserActivity.empty();

  late int userId;
  late String module;
  late String action;
  Map<String, dynamic>? extras;
  late int status;
  late DateTime time;

  NUIUserActivity setActive() {
    status = STATUS_SUCCESS;
    return this;
  }

  NUIUserActivity setInactive() {
    status = STATUS_ERROR;
    return this;
  }

  @override
  NUIEntMapper<NUIUserActivity> mapper() => NUIUserActivityEntity();
}
class NUIUserActivityEntity extends NUIEntMapper<NUIUserActivity>{
  @override
  String entityName() => "NUIUserActivity";

  @override
  List<NUIEntField> fields() {
    return [
      NUIEntField<NUIUserActivity>(name: "userId", type: NUIEntType.INTEGER, setter: (data, value) => data.userId = value, getter: (data) => data.userId),
      NUIEntField<NUIUserActivity>(name: "module", type: NUIEntType.STRING, setter: (data, value) => data.module = value, getter: (data) => data.module),
      NUIEntField<NUIUserActivity>(name: "action", type: NUIEntType.STRING, setter: (data, value) => data.action = value, getter: (data) => data.action),
      NUIEntField<NUIUserActivity>(name: "extras", type: NUIEntType.MAP, setter: (data, value) => data.extras = value, getter: (data) => data.extras),
      NUIEntField<NUIUserActivity>(name: "status", type: NUIEntType.INTEGER, setter: (data, value) => data.status = value, getter: (data) => data.status),
      NUIEntField<NUIUserActivity>(name: "time", type: NUIEntType.DATE, setter: (data, value) => data.time = asDateTime(value) as DateTime, getter: (data) => data.time),
    ];
  }

  @override
  NUIUserActivity newInstance() => NUIUserActivity.empty();
}
